/*
   Author: rsn8887 (based on TheMaister)
   License: Public domain

   This is an integer prescale filter that should be combined
   with a bilinear hardware filtering (GL_BILINEAR filter or some such) to achieve
   a smooth scaling result with minimum blur. This is good for pixelgraphics
   that are scaled by non-integer factors.
*/

float4x4 WorldViewProjection	: WORLDVIEWPROJECTION;

float2 texture_size : SOURCEDIMS;

texture source_tex 	: SOURCETEXTURE;

sampler	sourceSampler = sampler_state
{
	Texture   = (source_tex);
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

string combineTechnique : COMBINETECHNIQUE =  "Sharp_Bilinear_Simple";

struct VS_OUTPUT
{
	float4 oPosition 	: POSITION;
	float2 oTexCoord 	: TEXCOORD0;
	float2 texelScaled : TEXCOORD1;
};

// Vertex Shader

VS_OUTPUT vShader (float3 pos : POSITION, float2 texCoord : TEXCOORD0)
{
	VS_OUTPUT OUT = (VS_OUTPUT)0;

	OUT.oPosition = mul(float4(pos,1),WorldViewProjection);
	OUT.oTexCoord = texCoord;
	OUT.texelScaled = texCoord*texture_size;
			
	return OUT;
}

// Pixel Shader

float4 pShader ( in VS_OUTPUT inp, in float2 screenSpace : VPOS ) : COLOR
{  
	//find optimum integer prescale
	float2 prescaleFactor = floor(screenSpace/inp.texelScaled);	
	
	//inp.texelScaled has already been multiplied by texture_size inside vertex shader
	float2 texel_floored = floor(inp.texelScaled);
	float2 s = frac(inp.texelScaled);

	float2 region_range = 0.5 - 0.5 / prescaleFactor;

	// Figure out where in the texel to sample to get correct pre-scaled bilinear.
	// Uses the hardware bilinear interpolator to avoid having to sample 4 times manually. 
	float2 center_dist = s - 0.5;
	float2 f = (center_dist - clamp(center_dist, -region_range, region_range)) * prescaleFactor + 0.5;
	float2 mod_texel = texel_floored + f;
   
	return float4(tex2D(sourceSampler, mod_texel/texture_size).rgb, 1.0);   
}

technique Sharp_Bilinear_Simple
{
   pass P0
   {
     VertexShader = compile vs_3_0 vShader();
     PixelShader  = compile ps_3_0 pShader();
   }  
}